@@ -185,6 +185,7 @@ Glib::RefPtr<Gnome::Gda::DataModel> Base_DB::query_execute(const Glib::ustring&
       std::cout << "Debug: query string could not be converted to std::cout: " << ex.what() << std::endl;
     }
     */
+    
 
     result = gda_connection->execute_single_command(strQuery);
     if(!result)
@@ -896,6 +897,79 @@ void Base_DB::set_database_preferences(const SystemPrefs& prefs)
     get_document()->set_database_title(prefs.m_name);
 }
 
+bool Base_DB::create_table_with_default_fields(const Glib::ustring& table_name)
+{
+  if(table_name.empty())
+    return false;
+  
+  bool created = false;
+
+  //Primary key:
+  sharedptr<Field> field_primary_key(new Field());
+  field_primary_key->set_name(table_name + "_id");
+  field_primary_key->set_title(table_name + " ID");
+  field_primary_key->set_primary_key();
+  field_primary_key->set_auto_increment();
+
+  Gnome::Gda::FieldAttributes field_info = field_primary_key->get_field_info();
+  field_info.set_allow_null(false);
+  field_primary_key->set_field_info(field_info);
+
+  field_primary_key->set_glom_type(Field::TYPE_NUMERIC);
+  //std::cout << "field_primary_key->get_auto_increment():" << field_primary_key->get_auto_increment() << std::endl;
+
+  type_vecFields fields;
+  fields.push_back(field_primary_key);
+
+  //Description:
+  sharedptr<Field> field_description(new Field());
+  field_description->set_name("description");
+  field_description->set_title(_("Description")); //Use a translation, because the original locale will be marked as non-English if the current locale is non-English.
+  field_description->set_glom_type(Field::TYPE_TEXT);
+  fields.push_back(field_description);
+
+  //Comments:
+  sharedptr<Field> field_comments(new Field());
+  field_comments->set_name("comments");
+  field_comments->set_title(_("Comments"));
+  field_comments->set_glom_type(Field::TYPE_TEXT);
+  field_comments->m_default_formatting.set_text_format_multiline();
+  fields.push_back(field_comments);
+
+  sharedptr<TableInfo> table_info(new TableInfo());
+  table_info->set_name(table_name);
+  table_info->set_title( Utils::title_from_string( table_name ) ); //Start with a title that might be appropriate.
+
+  created = create_table(table_info, fields);
+
+    //Create a table with 1 "ID" field:
+   //MSYQL:
+    //query_execute( "CREATE TABLE \"" + table_name + "\" (" + primary_key_name + " INT NOT NULL AUTO_INCREMENT PRIMARY KEY)" );
+    //query_execute( "INSERT INTO \"" + table_name + "\" VALUES (0)" );
+
+    //PostgresSQL:
+    //query_execute( "CREATE TABLE " + table_name + " (\"" + primary_key_name + "\" serial NOT NULL  PRIMARY KEY)" );
+
+    //query_execute( "CREATE TABLE \"" + table_name + "\" (" +
+    //  field_primary_key->get_name() + " numeric NOT NULL  PRIMARY KEY," + 
+    //  extra_field_description + "varchar, " +
+    //  extra_field_comments + "varchar" +
+    //  ")" );
+
+  if(created)
+  {
+    //Save the changes in the document:
+    Document_Glom* document = get_document();
+    if(document)
+    {
+      document->add_table(table_info);
+      document->set_table_fields(table_info->get_name(), fields);
+    }
+  }
+
+  return created;
+}
+
 bool Base_DB::create_table(const sharedptr<const TableInfo>& table_info, const Document_Glom::type_vecFields& fields_in) const
 {
   //std::cout << "Base_DB::create_table(): " << table_info->get_name() << ", title=" << table_info->get_title() << std::endl;
@@ -906,6 +980,7 @@ bool Base_DB::create_table(const sharedptr<const TableInfo>& table_info, const D
   Document_Glom::type_vecFields fields = fields_in;
 
   //Create the standard field too:
+  //(We don't actually use this yet)
   if(std::find_if(fields.begin(), fields.end(), predicate_FieldHasName<Field>(GLOM_STANDARD_FIELD_LOCK)) == fields.end())
   {
     sharedptr<Field> field = sharedptr<Field>::create();
@@ -988,6 +1063,8 @@ bool Base_DB::insert_example_data(const Glib::ustring& table_name) const
     return true;
   }
 
+  //std::cout << "debug: inserting example_rows for table: " << table_name << std::endl;
+
   bool insert_succeeded = true;
 
 
@@ -1015,7 +1092,7 @@ bool Base_DB::insert_example_data(const Glib::ustring& table_name) const
 
   //Actually insert the data:
   const type_vecStrings vec_rows = Utils::string_separate(example_rows, "\n", false /* ignore \n inside quotes. */);
-  //std::cout << "debug: Base_DB::insert_example_data(): number of rows of data: " << vec_rows.size() << std::endl;
+  //std::cout << "  debug: Base_DB::insert_example_data(): number of rows of data: " << vec_rows.size() << std::endl;
 
   for(type_vecStrings::const_iterator iter = vec_rows.begin(); iter != vec_rows.end(); ++iter)
   {
@@ -1166,7 +1243,7 @@ sharedptr<LayoutItem_Text> Base_DB::offer_textobject(const sharedptr<LayoutItem_
         dialog->set_transient_for(*transient_for);
 
       dialog->set_textobject(start_textobject, Glib::ustring());
-      const int response = dialog->run();
+      int response = Glom::Utils::dialog_run_with_help(dialog, "window_textobject");
       dialog->hide();
       if(response == Gtk::RESPONSE_OK)
       {
@@ -1201,7 +1278,7 @@ sharedptr<LayoutItem_Image> Base_DB::offer_imageobject(const sharedptr<LayoutIte
         dialog->set_transient_for(*transient_for);
 
       dialog->set_imageobject(start_imageobject, Glib::ustring());
-      const int response = dialog->run();
+      const int response = Glom::Utils::dialog_run_with_help(dialog, "window_imageobject");
       dialog->hide();
       if(response == Gtk::RESPONSE_OK)
       {
@@ -1237,7 +1314,7 @@ sharedptr<LayoutItem_Notebook> Base_DB::offer_notebook(const sharedptr<LayoutIte
 
       dialog->set_notebook(start_notebook);
       //dialog->set_transient_for(*this);
-      const int response = dialog->run();
+      const int response = Glom::Utils::dialog_run_with_help(dialog, "dialog_notebook");
       dialog->hide();
       if(response == Gtk::RESPONSE_OK)
       {
@@ -1302,7 +1379,7 @@ Glib::ustring Base_DB::get_find_where_clause_quick(const Glib::ustring& table_na
 
       if(use_this_field)
       {
-        strClausePart = table_name + "." + field->get_name() + " " + field->sql_find_operator() + " " +  field->sql_find(quick_search);
+        strClausePart = "\"" + table_name + "\".\"" + field->get_name() + "\" " + field->sql_find_operator() + " " +  field->sql_find(quick_search);
       }
 
       if(!strClausePart.empty())
@@ -1405,16 +1482,16 @@ sharedptr<Field> Base_DB::get_field_primary_key_for_table(const Glib::ustring& t
   return sharedptr<Field>();
 }
 
-void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vecFields& all_db_fields, const sharedptr<const LayoutGroup>& group, Base_DB::type_vecLayoutFields& vecFields) const
+void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vecFields& all_db_fields, const sharedptr<LayoutGroup>& group, Base_DB::type_vecLayoutFields& vecFields) const
 {
   //g_warning("Box_Data::get_table_fields_to_show_for_sequence_add_group(): table_name=%s, all_db_fields.size()=%d, group->name=%s", table_name.c_str(), all_db_fields.size(), group->get_name().c_str());
 
-  LayoutGroup::type_map_const_items items = group->get_items();
-  for(LayoutGroup::type_map_const_items::const_iterator iterItems = items.begin(); iterItems != items.end(); ++iterItems)
+  LayoutGroup::type_map_items items = group->get_items();
+  for(LayoutGroup::type_map_items::iterator iterItems = items.begin(); iterItems != items.end(); ++iterItems)
   {
-    sharedptr<const LayoutItem> item = iterItems->second;
+    sharedptr<LayoutItem> item = iterItems->second;
 
-    sharedptr<const LayoutItem_Field> item_field = sharedptr<const LayoutItem_Field>::cast_dynamic(item);
+    sharedptr<LayoutItem_Field> item_field = sharedptr<LayoutItem_Field>::cast_dynamic(item);
     if(item_field)
     {
       //Get the field info:
@@ -1426,7 +1503,7 @@ void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustrin
         sharedptr<Field> field = get_fields_for_table_one_field(item_field->get_table_used(table_name), item->get_name());
         if(field)
         {
-          sharedptr<LayoutItem_Field> layout_item = glom_sharedptr_clone(item_field); //TODO_Performance: Reduce the copying.
+          sharedptr<LayoutItem_Field> layout_item = item_field;
           layout_item->set_full_field_details(field); //Fill in the full field information for later.
 
 
@@ -1449,7 +1526,7 @@ void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustrin
         //If the field does not exist anymore then we won't try to show it:
         if(iterFind != all_db_fields.end() )
         {
-          sharedptr<LayoutItem_Field> layout_item = glom_sharedptr_clone(item_field); //TODO_Performance: Reduce the copying here.
+          sharedptr<LayoutItem_Field> layout_item = item_field;
           layout_item->set_full_field_details(*iterFind); //Fill the LayoutItem with the full field information.
 
           //std::cout << "get_table_fields_to_show_for_sequence_add_group(): name=" << layout_item->get_name() << std::endl;
@@ -1464,10 +1541,10 @@ void Base_DB::get_table_fields_to_show_for_sequence_add_group(const Glib::ustrin
     }
     else
     {
-      sharedptr<const LayoutGroup> item_group = sharedptr<const LayoutGroup>::cast_dynamic(item);
+      sharedptr<LayoutGroup> item_group = sharedptr<LayoutGroup>::cast_dynamic(item);
       if(item_group)
       {
-        sharedptr<const LayoutItem_Portal> item_portal = sharedptr<const LayoutItem_Portal>::cast_dynamic(item);
+        sharedptr<LayoutItem_Portal> item_portal = sharedptr<LayoutItem_Portal>::cast_dynamic(item);
         if(!item_portal) //Do not recurse into portals. They are filled by means of a separate SQL query.
         {
           //Recurse:
@@ -1547,7 +1624,7 @@ Base_DB::type_vecLayoutFields Base_DB::get_table_fields_to_show_for_sequence(con
       //We will show the fields that the document says we should:
       for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroupSequence.begin(); iter != mapGroupSequence.end(); ++iter)
       {
-        sharedptr<const LayoutGroup> group = iter->second;
+        sharedptr<LayoutGroup> group = iter->second;
 
         if(true) //!group->m_hidden)
         {
@@ -1699,7 +1776,10 @@ void Base_DB::calculate_field(const LayoutFieldInRecord& field_in_record)
         sharedptr<const Field> field = refCalcProgress.m_field;
         if(field)
         {
-          refCalcProgress.m_value = glom_evaluate_python_function_implementation(field->get_glom_type(), field->get_calculation(), field_values, get_document(), field_in_record.m_table_name);
+          //We need the connection when we run the script, so that the script may use it.
+          sharedptr<SharedConnection> sharedconnection = connect_to_server(0 /* parent window */);
+
+          refCalcProgress.m_value = glom_evaluate_python_function_implementation(field->get_glom_type(), field->get_calculation(), field_values, get_document(), field_in_record.m_table_name, sharedconnection->get_gda_connection());
 
           refCalcProgress.m_calc_finished = true;
           refCalcProgress.m_calc_in_progress = false;
@@ -2232,6 +2312,134 @@ bool Base_DB::check_entered_value_for_uniqueness(const Glib::ustring& table_name
     return true; //Succeed, because the value does not need to be unique.
 }
 
+bool Base_DB::get_relationship_exists(const Glib::ustring& table_name, const Glib::ustring& relationship_name)
+{
+  Document_Glom* document = get_document();
+  if(document)
+  { 
+    sharedptr<Relationship> relationship = document->get_relationship(table_name, relationship_name);
+    if(relationship)
+      return true;
+  }
+ 
+  return false;
+}
+
+sharedptr<const LayoutItem_Field> Base_DB::get_field_is_from_non_hidden_related_record(const sharedptr<const LayoutItem_Portal>& portal) const
+{
+  //Find the first field that is from a non-hidden related table.
+  sharedptr<LayoutItem_Field> result;
+
+  if(!portal)
+    return result;
+
+  const Document_Glom* document = get_document();
+  if(!document)
+    return result;
+
+  LayoutItem_Portal::type_map_const_items items = portal->get_items(); 
+  for(LayoutItem_Portal::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
+  {
+    sharedptr<const LayoutItem_Field> field = sharedptr<const LayoutItem_Field>::cast_dynamic(iter->second);
+    if(field)
+    {
+      if(field->get_has_relationship_name())
+      {
+        const Glib::ustring table_name = field->get_table_used( portal->get_relationship()->get_to_table() );
+        if(!(document->get_table_is_hidden(table_name)))
+          return field;
+      }
+
+    }
+  }
+
+  return result;
+}
+
+sharedptr<const LayoutItem_Field> Base_DB::get_field_identifies_non_hidden_related_record(const sharedptr<const LayoutItem_Portal>& portal, sharedptr<const Relationship>& used_in_relationship) const
+{
+  //Find the first field that is from a non-hidden related table.
+  sharedptr<LayoutItem_Field> result;
+
+  const Document_Glom* document = get_document();
+  if(!document)
+    return result;
+
+  LayoutItem_Portal::type_map_const_items items = portal->get_items(); 
+  for(LayoutItem_Portal::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
+  {
+    sharedptr<const LayoutItem_Field> field = sharedptr<const LayoutItem_Field>::cast_dynamic(iter->second);
+    if(field && !(field->get_has_relationship_name()))
+    {
+      sharedptr<Relationship> relationship = document->get_field_used_in_relationship_to_one(portal->get_relationship()->get_to_table(), field->get_name());
+      if(relationship)
+      {
+        const Glib::ustring table_name = relationship->get_to_table();
+        if(!(table_name.empty()))
+        {
+          if(!(document->get_table_is_hidden(table_name)))
+          {
+            used_in_relationship = relationship;
+            return field;
+          }
+        }
+      }
+    }
+  }
+
+  return result;
+}
+
+sharedptr<const UsesRelationship> Base_DB::get_portal_navigation_relationship_automatic(const sharedptr<LayoutItem_Portal>& portal, bool& navigation_main) const
+{
+  //Initialize output parameters:
+  navigation_main = false;
+
+  const Document_Glom* document = get_document();
+  
+  //If the related table is not hidden then we can just navigate to that:
+  const Glib::ustring direct_related_table_name = portal->get_relationship()->get_to_table(); 
+  if(!(document->get_table_is_hidden(direct_related_table_name)))
+  {
+    //Non-hidden tables can just be shown directly. Navigate to it: 
+    navigation_main = true;
+    return sharedptr<UsesRelationship>();
+  }
+  else
+  {
+    //If the related table is hidden, 
+    //then find a suitable related non-hidden table by finding the first layout field that mentions one:
+    sharedptr<const LayoutItem_Field> field = get_field_is_from_non_hidden_related_record(portal);
+    if(field)
+    {
+      return field; //Returns the relationship part. (A relationship belonging to the portal's related table.)
+      //sharedptr<UsesRelationship> result = sharedptr<UsesRelationship>::create();
+      //result->set_relationship( portal->get_relationship() );
+      //result->set_related_relationship( field->get_relationship() );
+
+      //return result;
+    }
+    else
+    {
+      //Instead, find a key field that's used in a relationship, 
+      //and pretend that we are showing the to field as a related field:
+      sharedptr<const Relationship> used_in_relationship;
+      sharedptr<const LayoutItem_Field> field_identifies = get_field_identifies_non_hidden_related_record(portal, used_in_relationship);
+      if(field_identifies)
+      {
+        sharedptr<UsesRelationship> result = sharedptr<UsesRelationship>::create();
+       
+        sharedptr<Relationship> rel_nonconst = sharedptr<Relationship>::cast_const(used_in_relationship);
+        result->set_relationship(rel_nonconst);
+
+        return result;
+      }
+    }
+  }
+
+  //There was no suitable related table to show:
+  return sharedptr<UsesRelationship>(); 
+}
 
 bool Base_DB::get_primary_key_is_in_foundset(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value)
 {
